/* opto_robot

   Falkövető robot, amely a falat és az akadályokat két TCRT5000
   szenzorral érzékeli.

   A program a baloldali szenzor jelzései alapján követi a falat, a
   szenzor által érzékelt értéket az L_LIMIT és L_LIMIT_MIN között
   tartva. Mindeközben az elülső (front) szenzor a kikerülendő
   akadályokat észleli. Akadály észlelése eseténa robot jobbra fordul,
   amíg az elülső szenzor jele F_LIMIT_MIN alá nem csökken.

   A program "finomhangolását" egy nyomógomb és egy SSD1306 I2c kijelző
   segítségével végezhetjük el.
   RESET után, ha a gomb le van nyomva, egymás után sorra vesszük
   az l_limit, l_limit_min, f_limit és f_limit_min paramétereket és a
   robot falhoz/akadályhoz közelítésével beállíthatjuk a kívánt távolságot,
   majd a nyomógombbal rögzítjük az értéket. 
   
   Tested with Arduino IDE 1,8,13
*/
#include <Adafruit_SSD1306.h>

#define B1A 7          // Bal motor előre
#define B1B 6          // Bal motor hátra
#define A1A 5          // Jobb motor előre
#define A1B 4          // Jobb motor hátra

#define PUSH2 2        // Nyomógomb
#define L_sense A0
#define L_LED   10
#define F_sense A1
#define F_LED    9
#define NMEAS   10

#define SCREEN_WIDTH 128    // OLED kijelző szélessége képpontokban
#define SCREEN_HEIGHT 32    // OLED kijelző magassága képpontokban
#define OLED_RESET     -1   // -1 jelentése: nincs RESET kivezetés
#define SCREEN_ADDRESS 0x3C // A kijelző I2C címe (jobbra igazítva)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

boolean waitforpress = true;
uint16_t l_limit = 80;
uint16_t l_limit_min = 50;
uint16_t f_limit = 50;
uint16_t f_limit_min = 30;
uint16_t chksum;

uint32_t meres(int p_sense, int p_LED) {
  uint32_t sum0, sum1;
  digitalWrite(p_LED, LOW);
  sum0 = 0;
  for (int i = 0; i < NMEAS; i++) {
    sum0 += analogRead(p_sense);
  }
  digitalWrite(p_LED, HIGH);
  sum1 = 0;
  for (int i = 0; i < NMEAS; i++) {
    sum1 += analogRead(p_sense);
  }
  if (sum0 > sum1) sum0 = sum1;
  return ((sum1 - sum0) / NMEAS);
}

uint32_t get_parameter(char* txt, int p_sense, int p_led) {
  uint32_t mypar;
  while (waitforpress) {           // Ha lenyomásra várunk és
    mypar = meres(p_sense, p_led);
    display.clearDisplay();
    display.setTextSize(2);             // Normal 2:1 pixel scale
    display.setTextColor(SSD1306_WHITE);        // Draw white text
    display.setCursor(0, 0);            // Start at top-left corner
    display.println(txt);
    display.setCursor(0, 16);            // Start at middle-left position
    display.println(mypar);
    display.display();
    if (!digitalRead(PUSH2)) {   // Ha lenyomás történt...
      waitforpress = false;      // Következő stáció: felengedésre várunk
    }
  }
  delay(20);                     // Pergésmentesítő késleltetés
  while (!waitforpress) {
    if (digitalRead(PUSH2)) {    // Ha felengedést észlelünk...
      waitforpress = true;       // Következő stáció: lenyomásra várunk
    }
  }
  delay(20);                     // Pergésmentesítő késleltetés
  return mypar;
}

void setup() {
  analogReference(DEFAULT);
  pinMode(PUSH2, INPUT_PULLUP);
  pinMode(L_LED, OUTPUT);
  pinMode(F_LED, OUTPUT);
  // Motorvezérlő lábak konfigurálása
  pinMode(B1A, OUTPUT);
  pinMode(B1B, OUTPUT);
  pinMode(A1A, OUTPUT);
  pinMode(A1B, OUTPUT);
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, LOW);
  display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS);
  display.display();
  if (!digitalRead(PUSH2)) {
    while (!digitalRead(PUSH2));
    delay(20);
    l_limit = get_parameter("L_MAX", L_sense, L_LED);
    l_limit_min = get_parameter("L_MIN", L_sense, L_LED);
    f_limit = get_parameter("F_MAX", F_sense, F_LED);
    f_limit_min = get_parameter("F_MIN", F_sense, F_LED);
  }
  display.setTextSize(2);                 // Normal 2:1 pixel scale
  display.setTextColor(SSD1306_WHITE);    // Draw white text
  display.clearDisplay();
  display.setCursor(0, 0);
  display.print(l_limit);
  display.setCursor(64, 0);
  display.print(f_limit);
  display.setCursor(0, 16);
  display.print(l_limit_min);
  display.setCursor(64, 16);
  display.print(f_limit_min);
  display.display();
  delay(5000);
}

void loop() {
  motor_forward();
  uint32_t left = meres(L_sense, L_LED);
  uint32_t front = meres(F_sense, F_LED);

  if (left > l_limit) {
    motor_right();
  }
  else if (left < l_limit_min) {
    motor_left();
  }
  else {
    motor_forward();
  }

  if (front > f_limit) {
    while (front > f_limit_min) {
      front = meres(F_sense, F_LED);
      motor_right();
    }
  } else {
    delay(10);
  }
}

void motor_forward() {
  digitalWrite(B1A, HIGH);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, HIGH);
  digitalWrite(A1B, LOW);
}

void motor_reverse() {
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, HIGH);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, HIGH);
}

void motor_left() {
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, HIGH);
  digitalWrite(A1A, HIGH);
  digitalWrite(A1B, LOW);
}

void motor_right() {
  digitalWrite(B1A, HIGH);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, HIGH);
}

void motor_stop() {
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, LOW);
}
